"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const react_native_1 = require("react-native");
const Manager_1 = require("./Manager");
exports.Context = React.createContext(null);
class Host extends React.Component {
    constructor() {
        super(...arguments);
        this.nextKey = 0;
        this.queue = [];
        this.manager = null;
        this.mount = (children) => {
            const key = this.nextKey++;
            if (this.manager) {
                this.manager.mount(key, children);
            }
            else {
                this.queue.push({ type: 'mount', key, children });
            }
            return key;
        };
        this.update = (key, children) => {
            if (this.manager) {
                this.manager.update(key, children);
            }
            else {
                const op = { type: 'mount', key, children };
                const index = this.queue.findIndex(o => o.type === 'mount' || (o.type === 'update' && o.key === key));
                if (index > -1) {
                    this.queue[index] = op;
                }
                else {
                    this.queue.push(op);
                }
            }
        };
        this.unmount = (key) => {
            if (this.manager) {
                this.manager.unmount(key);
            }
            else {
                this.queue.push({ type: 'unmount', key });
            }
        };
    }
    componentDidMount() {
        const manager = this.manager;
        const queue = this.queue;
        while (queue.length && manager) {
            const action = queue.pop();
            if (action) {
                switch (action.type) {
                    case 'mount':
                        manager.mount(action.key, action.children);
                        break;
                    case 'update':
                        manager.update(action.key, action.children);
                        break;
                    case 'unmount':
                        manager.unmount(action.key);
                        break;
                }
            }
        }
    }
    render() {
        const { children, style } = this.props;
        return (React.createElement(exports.Context.Provider, { value: { mount: this.mount, update: this.update, unmount: this.unmount } },
            React.createElement(react_native_1.View, { style: [{ flex: 1 }, style], collapsable: false, pointerEvents: "box-none" }, children),
            React.createElement(Manager_1.Manager, { ref: el => (this.manager = el) })));
    }
}
exports.Host = Host;
